<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2016 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *
 * $Id: adtrack-http.php 379 2016-04-26 23:03:46Z shaun $
 *
 * This is the Adwiz HTTP handler. It proxies API requests from clients to the
 * Adwiz API.
 *
 * This is an API that runs on the installer server so it shares much with
 * the core API.
 */

handleRequest();
exit;

//! Request handler.
/*!
 * @return void
 *
 * SIDE EFFECT: may exit, redirect, or output to stdout
 */
function handleRequest() {
    $HTTPMethod = $_SERVER['REQUEST_METHOD'];

    // Currently I am only supporting GET and POST. We expand this as necessary.
    if ($HTTPMethod !== 'GET' && $HTTPMethod !== 'POST') {
        requestInvalid(i18n("Unsupported HTTP method"));
        return;
    }

    if (!array_key_exists('REMOTE_ADDR', $_SERVER) ||
        !is_string($_SERVER['REMOTE_ADDR'])) {
        requestInvalid(i18n("Unable to determine your IP address."));
        return;
    }

    $call = NULL;

    if ($HTTPMethod === 'GET') {
        if (array_key_exists('action', $_GET) &&
            is_string($_GET['action']) &&
            strlen($_GET['action']) > 0) {
            $call = $_GET['action'];
        }
    }

    if ($HTTPMethod === 'POST') {
        if (array_key_exists('action', $_POST) &&
            is_string($_POST['action']) &&
            strlen($_POST['action']) > 0) {
            $call = $_POST['action'];
        }
    }

    if ($call === NULL) {
        requestInvalid(i18n("You must specify an API call."));
        return;
    }

    // Gather config data.
    $config = readConfig();
    if ($config === FALSE) {
        requestInvalid(i18n("Configuration file has a problem."));
        return;
    }

    // Route it.

    if ($HTTPMethod === 'POST' && $call === 'init-app-box') {
        requestInitAppBox($config);
        return;
    }

    // TODO: Not currently supported.
    //if ($HTTPMethod === 'POST' && $call === 'event') {
    //    requestEvent($config);
    //    return;
    //}

    requestInvalid(i18n("Invalid request. API call not found."));
}

//! Read in our configuration file.
/*!
 * @return mixed Associative array or boolean false
 *   Keys: api_token, api_url
 */
function readConfig() {
    // We expect the config is in the same directory.
    $config = 'adtrack.conf.php';
    if (!file_exists($config)) {
        adwizLogError("Config file not found.");
        return FALSE;
    }

    // Sanity check size before reading it all in.
    $fileSize = @filesize($config);
    if ($fileSize === FALSE) {
        adwizLogError("Unable to read configuration file size.");
        return FALSE;
    }

    if ($fileSize >= 5*1024*1024) {
        adwizLogError("Config file is too large: $fileSize bytes.");
        return FALSE;
    }

    $contents = @file_get_contents($config);
    if ($contents === FALSE) {
        adwizLogError("Unable to read in configuration file.");
        return FALSE;
    }

    $keys = array();
    $lines = explode("\n", $contents);
    foreach ($lines as $line) {
        $line = trim($line);

        if (strlen($line) === 0) {
            continue;
        }

        $commentPos = strpos($line, '#');
        if ($commentPos === 0) {
            continue;
        }

        if ($commentPos !== FALSE) {
            $line = substr($line, 0, $commentPos);
        }

        if (!preg_match('/^(.+?)\s*=\s*(.*)$/', $line, $matches)) {
            adwizLogError("Malformed line in configuration file: $line");
            continue;
        }

        $key = $matches[1];
        $value = $matches[2];

        if (array_key_exists($key, $keys)) {
            adwizLogError("Key found twice. Ignoring. ($key)");
            continue;
        }

        // Yes, value may be blank.
        $keys[$key] = $value;
    }

    $requiredKeys = array('api_token', 'api_url');

    foreach ($requiredKeys as $key) {
        if (!array_key_exists($key, $keys) ||
            strlen($keys[$key]) === 0) {
            adwizLogError("Missing required key in config file. ($key)");
            return FALSE;
        }
    }

    return $keys;
}

//! API call: init-app-box
/*!
 * @param array $config See readConfig().
 *
 * @returns void
 *
 * POST /v1/init-app-box
 */
function requestInitAppBox($config) {
    if (!is_array($config)) {
        requestInvalid(i18n("Invalid parameter to init-app-box."));
        return;
    }

    // Must have an id that tell us what app box to load.
    if (!array_key_exists('id', $_POST) ||
        !is_string($_POST['id']) ||
        !is_numeric($_POST['id']) ||
        $_POST['id'] > PHP_INT_MAX) {
        requestInvalid(i18n("You must provide an app box identifier."));
        return;
    }
    $appBoxId = intval($_POST['id']);

    adwizLogError("Adtrack request: " . $_SERVER['REMOTE_ADDR'] . ': init-app-box: ' . $appBoxId);


    // Pick up data gathering data passed to us.
    $data = gatherData();
    if ($data === FALSE) {
        adwizLogError("requestInitAppBox: Unable to gather data");
        requestInvalid(i18n("Unable to gather data."));
        return;
    }

    // Make API request.

    $url = $config['api_url'];
    // Request parameters.
    // Make sure our core ones override any from data (not that there should
    // be any overlap, but just in case).
    // NOTE: We don't need to URL encode parameters. curl does this.
    $params = array_merge($data,
        array(
            'version'   => '1',
            'action'    => 'init-app-box',
            'id'        => $appBoxId,
        )
    );
    $result = APIRequest($config, 'POST', $url, $params);
    if ($result === FALSE) {
        requestInvalid(i18n("Unable to make API request."));
        return FALSE;
    }

    echo $result;
}

//! Gather data we want to pass to the API.
/*!
 * @return mixed Associative array or bool FALSE if there was a problem.
 *
 * There is data we want to know about. It is not core to the functionality.
 * We gather this here and put into an array.
 */
function gatherData() {
    $headers = getHeaders();
    if ($headers === FALSE) {
        adwizLogError('gatherData: Unable to retrieve headers.');
        return FALSE;
    }

    // Pre condition: We checked REMOTE_ADDR is available.
    $clientIP = $_SERVER['REMOTE_ADDR'];

    // Server IP is our own. API finds this upon receipt of request.

    $userAgent = '';
    if (array_key_exists('user-agent', $headers)) {
        $userAgent = $headers['user-agent'];
    }

    $acceptLanguage = '';
    if (array_key_exists('accept-language', $headers)) {
        $acceptLanguage = $headers['accept-language'];
    }

    $referer = '';
    if (array_key_exists('referer', $headers)) {
        $referer = $headers['referer'];
    }


    $host = '';
    if (array_key_exists('host', $headers)) {
        $host = $headers['host'];
    }

    // API finds the time.

    // Space id comes from request we received (separate).

    // Ad id - API determines this.

    // Type comes from action we send (separate).

    $page = '';
    if (array_key_exists('page', $_POST) &&
        is_string($_POST['page'])) {
        $page = trim($_POST['page']);
    }

    $refererOri = '';
    if (array_key_exists('referer_ori', $_POST) && is_string($_POST['referer_ori'])) {
        $refererOri = trim($_POST['referer_ori']);
    }


    return array(
        'client-ip'       => $clientIP,
        'user-agent'      => $userAgent,
        'accept-language' => $acceptLanguage,
        'referer'         => $referer,
        'referer-ori'     => $refererOri,
        'host'            => $host,
        'page'            => $page,
    );
}

//! Retrieve headers from the request.
/*!
 * @return mixed Associative array or boolean FALSE if failure.
 *
 * We convert all header names to lower case.
 */
function getHeaders() {
    $headers = getallheaders();
    if ($headers === FALSE) {
        adwizLogError("getHeaders: Unable to retrieve headers.");
        return FALSE;
    }

    $headersLowered = array();
    foreach ($headers as $name => $value) {
        $headersLowered[strtolower($name)] = $value;
    }

    return $headersLowered;
}

//! API call: event
/*!
 * @param array $config See readConfig()
 *
 * @returns void
 *
 * POST /v1/event
 */
function requestEvent($config) {
    if (!is_array($config)) {
        requestInvalid(i18n("Invalid parameter to event."));
        return;
    }

    // Do minimal checks. Only that we have what we should. Pass directly
    // through to core API which does the main validation.

    $keys = array(
        'type',
        'app-box-id',
        'component-id',
    );

    foreach ($keys as $key) {
        if (!array_key_exists($key, $_POST) ||
            !is_string($_POST[$key]) ||
            strlen($_POST[$key]) === 0) {
            requestInvalid(i18n("Missing required parameter: $key"));
            return;
        }
    }

    // Make API request.

    $url = $config['api_url'];
    $params = array(
        'version'      => '1',
        'action'       => 'event',
        'type'         => $_POST['type'],
        'app-box-id'   => $_POST['app-box-id'],
        'component-id' => $_POST['component-id'],
    );

    $result = APIRequest($config, 'POST', $url, $params);
    if ($result === FALSE) {
        requestInvalid(i18n("Unable to make API request."));
        return FALSE;
    }
    adwizLogError("Sent API request with params " . print_r($params, TRUE));

    echo $result;
}

//! Make an HTTP API request.
/*!
 * @param array $config See readConfig()
 * @param string $verb HTTP verb
 * @param string $url
 * @param mixed $body Request body. Array if you want to pass any. Null if not.
 *
 * @return mixed String the response or bool false if we failed.
 */
function APIRequest($config, $verb, $url, $body) {
    if (!is_array($config)) {
        adwizLogError("Missing config information");
        return FALSE;
    }
    if (!is_string($verb) || strlen($verb) === 0) {
        adwizLogError("Missing HTTP verb.");
        return FALSE;
    }
    if (!is_string($url) || strlen($url) === 0) {
        adwizLogError("Missing URL");
        return FALSE;
    }

    // Body is optional.

    $ch = curl_init();
    if ($ch === FALSE) {
        adwizLogError("Unable to setup cURL handle.");
        return FALSE;
    }

    $setoptSuccess = TRUE;

    if (!curl_setopt($ch, CURLOPT_URL, $url)) {
        $setoptSuccess = FALSE;
    }

    if (!curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $verb)) {
        $setoptSuccess = FALSE;
    }

    if ($body !== NULL) {
        if (!curl_setopt($ch, CURLOPT_POSTFIELDS, $body)) {
            $setoptSuccess = FALSE;
        }
    }

    if (!curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, TRUE)) {
        $setoptSuccess = FALSE;
    }

    if (!curl_setopt($ch, CURLOPT_HTTPHEADER,
            array(
                'Authorization: ' . $config['api_token'],
            ))) {
        $setoptSuccess = FALSE;
    }

    if (!curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE)) {
        $setoptSuccess = FALSE;
    }

    // Don't include headers in output.
    if (!curl_setopt($ch, CURLOPT_HEADER, FALSE)) {
        $setoptSuccess = FALSE;
    }

    // Timeouts

    if (!curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5)) {
        $setoptSuccess = FALSE;
    }

    if (!curl_setopt($ch, CURLOPT_TIMEOUT, 5)) {
        $setoptSuccess = FALSE;
    }

    if (!$setoptSuccess) {
        adwizLogError("Unable to set cURL options.");
        return FALSE;
    }

    $body = curl_exec($ch);
    if ($body === FALSE) {
        adwizLogError("Unable to connect.");
        curl_close($ch);
        return FALSE;
    }

    if (curl_errno($ch) !== 0) {
        adwizLogError("cURL error: " . curl_error($ch));
        curl_close($ch);
        return FALSE;
    }

    curl_close($ch);

    return $body;
}

//! Respond to a client who sent us an invalid request.
/*!
 * @param string $message A readable message about what is wrong.
 *
 * @return void
 *
 * SIDE EFFECT: Output to stdout.
 */
function requestInvalid($message) {
    if (!is_string($message) || strlen($message) === 0) {
        $message = i18n('Unknown error.');
    }

    $error = array(
        'message' => $message,
    );

    JSONSend($error);
}

//! Generate and send a JSON response.
/*!
 * @param array $response
 *
 * @return void
 */
function JSONSend($response) {
    if (!is_array($response)) {
        $response = array(
            'error' => i18n("JSON response sent with insufficient information."),
        );
    }

    $json = json_encode($response);

    if ($json === FALSE) {
        $json = jsonError(i18n("Unable to encode response."));
    }

    header('Content-Type: application/json');
    echo $json;
}

//! generate a json error object
/*!
 * @param string $msg
 *
 * @return string
 */
function JSONError($msg) {
    if (!is_string($msg) || strlen($msg) === 0) {
        adwizLogError('jsonError: missing message');
        $msg = i18n('Missing message');
    }

    // TODO: encoding
    return '{"error": "' . $msg . '"}';
}

//! Log a message.
/*!
 * @param string $msg
 *
 * @return void
 *
 * TODO: This function could use a rename. It need not be for only errors.
 *
 * TODO: This function opens and closes syslog each time. It is probably
 *   not efficient.
 */
function adwizLogError($msg) {
    if (!isset($msg)) {
        return;
    }
    if (!is_string($msg) &&
        !is_int($msg) &&
        !is_float($msg)) {
        return;
    }

    // Leave preceding whitespace. Indenting a bit can be nice.
    $msg = rtrim($msg);
    if (strlen($msg) === 0) {
        return;
    }

    if (!openlog('adtrack', LOG_ODELAY | LOG_PID, LOG_USER)) {
        return;
    }

    syslog(LOG_INFO, $msg);

    closelog();
}

//! Translate to displayed language
/*!
 * @param string $key
 *
 * @return string Translated
 *
 * Currently we just pass the key right back.
 */
function i18n($key) {
    return $key;
}
