<?php
/*
Plugin Name: AdTrack
Plugin URI:  https://www.adtrack.ca
Description: The easiest way to rent space on your Wordpress sites. Display and sell ads with full support for impression statistics. After you install and enable the plugin, use the new AdTrack menu option to configure the plugin. Steps that you need to take 1) Create an AdTrack account at <a href="https://www.adtrack.ca">AdTrack</a>, create and configure your rental spaces. 2) Get the Access Key and Security Code from your AdTrack account (Dashboard/Space/Embedding) in your site. 3) Insert the Access Key and Security Code in the AdTrack plugin module. 4) Final step from your AdTrack plugin module select a Wordpress "shortcode" matching the Rental Spaces you wish to display and paste them in your post or page.
Version:     1.1
Author:      Wizard
Author URI:  http://www.wizard.com
License:     Proprietary

 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2016-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 *
 */
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

// define keys where we store data on Wordpress options table
define('ADTRACK_API_TOKEN', 'adtrack_api_token_lm');
define('ADTRACK_API_SECURITY_CODE', 'adtrack_api_security_token_lm');
define('ADTRACK_APP_BOX_JSON', 'adtrack_app_box_json');
define('ADTRACK_NOTIFICATION_JSON', 'adtrack_notification_json');
define('ADTRACK_LAST_SYNC', 'adtrack_last_sync');
define('ADTRACK_FLASH_MESSAGE', 'adtrack_flash_message');
define('ADTRACK_OPTION_SETTINGS', 'adtrack_option_settings');


// Process any POST requests
adtrack_process_post_requests();

// Add AdTrack menu option to the Admin menu
add_action('admin_menu', 'adtrack_add_options_menu_page');

// Register hook to remove the data from Wordpress options
register_uninstall_hook(__FILE__, 'adtrack_uninstall');

// Load the CSS and JS on the admin interface
add_action('admin_enqueue_scripts', "adtrack_add_css_and_js_files");

// Hook to register the new Shortcode
add_shortcode('AdTrack', 'adtrack_run_shortcode');

// Do we want to include rental spaces on widgets text
$options = get_option(ADTRACK_OPTION_SETTINGS, array());
if (isset($options) && is_array($options) && isset($options['shortcode']) && $options['shortcode'] === TRUE) {
    add_filter( 'widget_text', 'shortcode_unautop');
    add_filter( 'widget_text', 'do_shortcode');
}

// ****************************************************************************************************** //
// Helper functions Section
// ****************************************************************************************************** //

//! Helper function that is comparing a hash by the 'time' key
/*
 * @param array $a element that we compare against element $b
 *        array $b element that we compare against element $a
 *
 * @return int 0 if they are the same -1 if $a['time'] is smaller 1 otherwise
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_cmp($a, $b) {
    if ($a['time'] === $b['time']) {
        return 0;
    }
    return ($a['time'] < $b['time']) ? -1 : 1;
}

//! Helper function that is returning HTML with span for unread notifications
/*
 * @return string with HTML, could be a empty string if there are no unread notifications
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_get_unread_notifiation_count() {

    $notificatioHTML = '';
    // get the notification from options
    $notifications = get_option(ADTRACK_NOTIFICATION_JSON, array());
    // Did we get any notifications
    if (is_array($notifications) && sizeof($notifications) > 0) {
        $count = 0;
        // loop and count only the ones that are unread and not ignored
        foreach ($notifications as $item) {
            if ($item['read'] === FALSE && (!isset($item['ignore']) || $item['ignore'] !== 1)) {
                $count++;
            }
        }
        // if there are notification then let's generate the HTML to display a bubble with a number
        if ($count > 0) {
            $notificatioHTML = '<span class="notification-count">' . $count . '</span>';
        }
    }
    return $notificatioHTML;
}

//! Helper function that is updating the notification to the Wordpress option
/*
 * @params int $boxId the boxId that we want to update (boxId === rental space Id)
 *         bool $found flag to which tell us if we found the boxId on the existing rental spaces list
 *         bool $suspend we found the boxId, but is was suspended from the interface
 *
 * @return bool whatever we updated the notification or not
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_update_notification($boxId, $found, $suspended) {
    if (!isset($boxId) || !isset($found) || !is_bool($found) || !isset($suspended) || !is_bool($suspended)) {
        return FALSE;
    }

    $changed = FALSE;
    // Nothing else to do
    if ($found === TRUE && $suspended === FALSE) {
        return TRUE;
    }
    // get the notification
    $notifications = get_option(ADTRACK_NOTIFICATION_JSON, array());
    // sane the notifications and set it as an array
    if (!is_array($notifications) || sizeof($notifications) == 0) {
        $notifications = array();
    }
    // is there already a notification with this ID?
    if (isset($notifications[$boxId])) {
        // Did the user choose to ignore this notification ???
        if (!isset($notifications[$boxId]['ignore']) || $notifications[$boxId]['ignore'] !== 1) {
            // If not then update the message, could be a new reason now
            if ($suspended) {
                $notifications[$boxId]['message'] = 'The rental space ID #' . $boxId . ' was found but it is suspended!';
            } else {
                $notifications[$boxId]['message'] = 'An invalid shortcode was found for the rental space ID #' . $boxId . '!<br /> Please update the rental space ID or remove the shortcode!';
            }
            // mark the notification as unread
            $notifications[$boxId]['read'] = FALSE;
            // Update the time
            $notifications[$boxId]['time'] = time();
            // set the flag as changed
            $changed = TRUE;
        } else {
            $changed = FALSE;
        }
    } else {
        // This is a new notification
        if ($suspended) {
            $notifications[$boxId]['message'] = 'The rental space ID #' . $boxId . ' was found on post, but it was suspended!';
        } else {
            $notifications[$boxId]['message'] = 'An invalid shortcode was found for the rental space ID #' . $boxId . '!<br /> Please update the rental space ID or remove the shortcode!';
        }
        $notifications[$boxId]['read'] = FALSE;
        $notifications[$boxId]['time'] = time();
        $changed = TRUE;
    }

    // Did we update the notifications hash ?
    if ($changed) {
        update_option(ADTRACK_NOTIFICATION_JSON, $notifications, TRUE);
    }
    return TRUE;
}

//! Api call to retrieve the account status and additional information
/*
 * @param string $authentication 64 characters authentication code
 *        string $securityCode 16 characters security code
 *
 * @return array holds the information, status and error message
 *
 * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
 */
function adtrack_api_call($authentication, $securityCode){
    // variables
    // API url
    $url = 'https://api.adtrack.ca/api.php';
    // postfields query
    $headers = array();
    // curl fields query
    $curlFields = array();
    // time limit
    $timeLimit = 10;
    // sleep time in second
    $sleepTime = 5;
    // number of attempts allowed
    $numOfAttempt = 3;
    // response of the api
    $response = '';
    // response code from the api call
    $responseCode = 0;
    // error log
    $error = '';

    // sanity check
    if (!isset($authentication) ||
        !is_string($authentication) ||
        strlen($authentication) != 64) {

        return array(
            'result-code' => 'error',
            'result-code-string' => 'The Access Key is not valid',
        );
    }
    if (!isset($securityCode) ||
        !is_string($securityCode) ||
        strlen($securityCode) != 16) {

        return array(
            'result-code' => 'error',
            'result-code-string' => 'The Security Code is not valid',
        );
    }

    // add additional headers
    $headers = array('Authorization:' . $authentication);

    // construct curl field query
    $curlFields = array(
        'client_security_code' => $securityCode,
        'version' => 1,
        'action' => 'get-account-info',
    );

    // try up to 3 times to retrieve the information
    for ($attempt = 0; $attempt < $numOfAttempt && $responseCode !== 200; $attempt++) {

        // retry in 5 seconds and reset error log if the previous attempt failed
        if ($attempt > 0) {
            sleep($sleepTime);

            // reset error log unless it's the last attempt
            $error = '';
        }

        // set time limit to 30 seconds;
        set_time_limit($sleepTime);

        // open connection
        $ch = curl_init();

        // curl set option section
        curl_setopt($ch, CURLOPT_TIMEOUT, $sleepTime);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $curlFields);
        curl_setopt($ch, CURLOPT_NOBODY, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, TRUE);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_HEADER, FALSE);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        // execute and get response
        $response = curl_exec($ch);

        // log any errors and retry if any error is present
        if (curl_errno($ch)) {
            $error .= 'There was a cUrl error: ' . curl_error($ch) . "\n";
            curl_close($ch);
            continue;
        }

        // get response code and close
        $responseCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        // return the information and status if call is successful 200 or make another attempt
        if ($responseCode === 200) {
            // validate $response
            if (!isset($response) || !is_string($response)) {
                $error .= 'Unexpected curl_exec response' . "\n";
                continue;
            }

            $result = json_decode($response, TRUE);

            // validate $result
            if (!isset($result) || !is_array($result) ||
                !isset($result['result-code']) || !is_string($result['result-code']) ||
                !isset($result['result-code-string']) || !is_string($result['result-code-string'])) {
                $error .= 'Unexpected json_decode result';
                continue;
            }
            return $result;
        } 
    } 

    $result = json_decode($response, TRUE);

    // validate $result
    if (!isset($result) || !is_array($result) ||
        !isset($result['message']) || !is_string($result['message'])) {
        $error .= 'Unexpected Server Response';
    } else {
        // try to include result
        $error .= 'Response code: ' . $responseCode . "\n";
        $error .= 'Response: ' . $result['message'] . "\n";
    }

    // return the status and error message
    return array(
        'result-code' => 'error',
        'result-code-string' => $error,
    );
}

//! brief create a flash message the will be stored in session, this message can be used on the next request
/*
 * @params string $message array of messages that we want to be displayed into the alert
 *         string the type of the alert it could be 'success', or 'error'
 *
 * @return bool TRUE is the message was set successful or FALSE otherwise
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 * Note: We expect the message is already i18n and html escaped
 */
function adtrack_set_flash_message($message, $type = 'success') {
    if (!$message || !is_string($message) || strlen($message) < 1) {
        return FALSE;
    }
    // try to save if the type is wrong go with default seccess
    if (!$type || !in_array($type, array('success', 'error'))) {
        $type = 'success';
    }

    update_option(ADTRACK_FLASH_MESSAGE, array($type => $message), TRUE);
    return TRUE;
}
//! brief get the flash message get generate the HTML and unset the session
/*
 * @return string with HTML message if everything is ok or empty string otherwise
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 * Note: we expect that message will be like $_SESSION['ADTRACK_FLASH_MESSAGE'] = array('error' or 'success' => $string_message);
 */
function adtrack_get_flash_message() {
    // check is there is a session message available
    $messages = get_option(ADTRACK_FLASH_MESSAGE, '');
    if (!isset($messages) || empty($messages)) {
        return '';
    }

    $keys = array_keys($messages);
    // we expect to have only one key 'error' or success
    if (!$keys || !is_array($keys) || sizeof($keys) !== 1) {
        return '';
    }
    if (!in_array($keys[0], array('success', 'error'))) {
        return '';
    }
    $type = $keys[0];
    $message = $messages[$type];
    // we expect the message to be a string
    if (!$message || !is_string($message) || strlen($message) == 0) {
        return '';
    }
    // let's start building the HTML
    $html = '';
    $html .= '<div id="adtrack-message">' . "\n";
    $html .= '  <p class="' . $type . '">' . $message . '</p>' . "\n";
    $html .= '</div><!-- div.' . $type . '-->' . "\n";
    // remove the message from session to not have the same message on refresh
    delete_option(ADTRACK_FLASH_MESSAGE);
    return $html;
}


// ****************************************************************************************************** //
// Callbacks process Section
// ****************************************************************************************************** //

//! Callback function that will start a session if not already started
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_start_session() {
    if (session_id() === '') {
        session_start();
        die('Here');
    }
}


//! Callback function that will end a session if started
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_end_session() {
    if (session_id()) {
        session_destroy ();
    }
}
//! Add a new menu option to Wordpress Admin menu
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_add_options_menu_page() {
    add_menu_page(
        'AdTrack', // Menu title page slug
        'AdTrack' . adtrack_get_unread_notifiation_count(), // page title
        'manage_options', // capabilities required to see the page
        'adtrack', // admin page slug
        'adtrack_ouput_page' // callback function to display the option page
    );
}

//! Callback function to uninstall and remove tables and or option from WP options
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_uninstall() {

    // Remove the options from Wordpress
    delete_option(ADTRACK_API_TOKEN);
    delete_option(ADTRACK_APP_BOX_JSON);
    delete_option(ADTRACK_API_SECURITY_CODE);
    delete_option(ADTRACK_NOTIFICATION_JSON);
    delete_option(ADTRACK_LAST_SYNC);
    delete_option(ADTRACK_FLASH_MESSAGE);
    delete_option(ADTRACK_OPTION_SETTINGS);
}

//! Callback function to add the CSS and JS to the admin interface
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_add_css_and_js_files(){
    wp_enqueue_script('adtrack.js', plugins_url('/js/adtrack.js', __FILE__), array(), '1.0.0', true);
    wp_enqueue_style('adtrack.css', plugins_url('/css/adtrack.css', __FILE__), false);
}

//! Callback function that is processing the shortcode
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_run_shortcode($attr) {
    if (!isset($attr) || !is_array($attr) || !isset($attr['id'])){
        return FALSE;
    }

    $boxId = $attr['id'];
    // get the rental spaces from the WP options
    $prevRentalSpaces = get_option(ADTRACK_APP_BOX_JSON, array());

    // Determine if this BoxId is valid and was not suspended from our interface
    $found = FALSE;
    $suspended = FALSE;
    if (is_array($prevRentalSpaces) || sizeof($prevRentalSpaces) > 0) {
        foreach ($prevRentalSpaces as $space) {
            if ($space['app_box_id'] == $boxId) {
                $found = TRUE;
                if (isset($space['suspend']) && $space['suspend'] === 1) {
                    $suspended = TRUE;
                }
                break;
            }
        }
    }
    // update the notification
    // this function will check if there is a reason to record a new notification
    adtrack_update_notification($boxId, $found, $suspended);

    // Don't want to process the short code if we did not found the box ID or the box Id was suspended
    if (!$found || $suspended) {
        return 'Not found';
    }

    // Load the JS only once
    wp_enqueue_script('adtrack-client.js', 'https://adtrack.ca/js/client.js', array(), '1.0.0', true);

    //get the plugin path to that the AJAX knows where to call
    $plugin_path = plugins_url();
    $html =<<<END
<script id="adtrack-{$boxId}" type="text/javascript">
    if (Adtrack === undefined) {
        var Adtrack = {};
        Adtrack.basedir = "{$plugin_path}/adTrack";
    }
    Adtrack.ToLoad = Adtrack.ToLoad || [];
    Adtrack.ToLoad.push({$boxId});
</script>
END;
    return $html;
}

//! Callback function that will generate the output for the Admin interface
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_ouput_page() {
    if (! isset($_REQUEST['settings-updated'])) {
        $_REQUEST['settings-updated'] = false;
    }

    $html ='';

    $html .= '<div class="wrap">' . "\n";
    $html .= '      <div id="adtrack">' . "\n";

    // look for flash messages and render if any
    $html .= adtrack_get_flash_message();

    $html .= '          <div id="post-body">' . "\n";
    $html .= '              <div id="post-body-content">' . "\n";

    $html .= '                      <div class="">' . "\n";

    $html .= '                          <img src="' . plugins_url('img/adtrack-mini-logo-alt.png', __FILE__) . '" alt="AdTrack Logo" class="alignright" />' . "\n";
    $html .= '                          <ul class="nav nav-tabs">' . "\n";
    $html .= '                              <li class="active" data-section="adtrack-dashboard" role="presentation"><a onclick="AT.showSection(\'adtrack-dashboard\', event);" href="#">Dashboard</a></li>' . "\n";
    $html .= '                              <li class="" data-section="adtrack-rental" role="presentation"><a href="#adtrack-rental" onclick="AT.showSection(\'adtrack-rental\', event);">Rental Spaces</a></li>' . "\n";
    $html .= '                              <li class="" data-section="adtrack-settings" role="presentation"><a href="#adtrack-settings" onclick="AT.showSection(\'adtrack-settings\', event);">AdTrack Settings</a></li>' . "\n";

    $html .= '                              <li class="" data-section="adtrack-notification" role="presentation"><a href="#adtrack-notification" onclick="AT.showSection(\'adtrack-notification\', event);">Notification' . adtrack_get_unread_notifiation_count() . '</a></li>' . "\n";
    $html .= '                              <li class=""  data-section="adtrack-help" role="presentation"><a href="#adtrack-help" onclick="AT.showSection(\'adtrack-help\', event);">Help</a></li>' . "\n";
    $html .= '                          </ul>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";

    $html .= adtrack_fragment_dashboard();
    $html .= adtrack_fragment_rental_space();
    $html .= adtrack_fragment_settings();
    $html .= adtrack_fragment_notification();
    $html .= adtrack_fragment_help();


    // This is the confirm box
    // Content can be populated by JS
    $html .= '<div id="adtrack_confirmOverlay" onclick="AT.hideConfirmBox(event)">' . "\n";
    $html .= '    <div id="adtrack_confirmBox">' . "\n";
    $html .= '    </div>' . "\n";
    $html .= '</div>' . "\n";
    // hidden fragment with the forms
    $html .= adtrack_update_api_token_fragment();
    $html .= '            </div> <!-- end post-body-content -->' . "\n";
    $html .= '       </div> <!-- end post-body -->' . "\n";
    $html .= '  </div> <!-- end poststuff -->' . "\n";
    $html .= '</div>' . "\n";

    print $html;
}


// ****************************************************************************************************** //
// POST process Section
// ****************************************************************************************************** //

//! Wrapper function that will process any POST request to update the system
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_process_post_requests() {
    if (!isset($_POST) || !isset($_POST['adtrack_action'])) {
        return '';
    }
    $action = trim($_POST['adtrack_action']);
    switch ($action) {
        case 'adtrack-update-api-token':
            adtrack_process_update_api_request();
        break;
        case 'adtrack-update-app-boxes':
            adtrack_process_update_app_boxes();
        break;
        case 'adtrack-notification-update':
            adtrack_process_notification_update();
        break;
        case 'adtrack-settings':
            adtrack_process_settings();
        break;
    }
}

//! POST processing function for updating the settings
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_process_settings() {
    if (!isset($_POST['shortcode']) || !is_string($_POST['shortcode'])) {
        return;
    }

    $options = get_option(ADTRACK_OPTION_SETTINGS, array());
    if (!isset($options) || !is_array($options)) {
        $options = array();
    }
    $newValue = trim($_POST['shortcode']) == '1' ? TRUE : FALSE;

    if (isset($options['shortcode']) && $options['shortcode'] === $newValue) {
        return adtrack_set_flash_message('Nothing changed!', 'error');
    }

    $options['shortcode'] = $newValue;
    if (update_option(ADTRACK_OPTION_SETTINGS, $options, TRUE) !== TRUE) {
        return adtrack_set_flash_message('Unable to store the new settings update!', 'error');
    }

    return adtrack_set_flash_message('Settings were updated successfully!', 'success');
}

//! POST processing function for updating the notification status (mark as read, ignore and delete)
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_process_notification_update() {
    $boxId = FALSE;
    $action = 'update';
    // for update let's store what do we need to changed
    $key = '';
    // and what value should we use now
    $value = '';
    if (isset($_POST['read']) && is_numeric($_POST['read'])) {
        $boxId = $_POST['read'];
        $key = 'read';
        $value = TRUE;
    }
    if (isset($_POST['unread']) && is_numeric($_POST['unread'])) {
        $boxId = $_POST['unread'];
        $key = 'read';
        $value = FALSE;
    }

    if (isset($_POST['ignore']) && is_numeric($_POST['ignore'])) {
        $boxId = $_POST['ignore'];
        $key = 'ignore';
        $value = 1;
    }

    if (isset($_POST['delete']) && is_numeric($_POST['delete'])) {
        $boxId = $_POST['delete'];
        $action = 'delete';
    }

    $notifications = get_option(ADTRACK_NOTIFICATION_JSON, array());
    if (!is_array($notifications) || sizeof($notifications) === 0 || !isset($notifications[$boxId])) {
        return adtrack_set_flash_message('Unable to fetch your notification', 'error');
    }
    if ($action === 'delete') {
        unset($notifications[$boxId]);
    } else {
        $notifications[$boxId][$key] = $value;
        // for ignore we want to make the notification as read
        if ($key === 'ignore') {
            $notifications[$boxId]['read'] = TRUE;
        }
    }

    // Update the notification on WP option
    if (update_option(ADTRACK_NOTIFICATION_JSON, $notifications, TRUE) === FALSE) {
        return adtrack_set_flash_message('Unable to update your notification', 'error');
    }

    return TRUE;

}

//! helper function the is validate and update the AdTrack API token
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_process_update_api_request() {
    if (!isset($_POST['api_token']) || empty($_POST['api_token']))  {
        return adtrack_set_flash_message('You must set a Access Key', 'error');
    }
    $token = trim($_POST['api_token']);


    if (!isset($_POST['security_code']) || empty($_POST['security_code']))  {
        return adtrack_set_flash_message('You must set a Security Code', 'error');
    }
    $securityCode = trim(strtolower($_POST['security_code']));
    // strip any chars that are not valid
    $securityCode = preg_replace("/[^a-z0-9]/", "", $securityCode);
    // Attempt to validate the token to make sure that the length is 64 chars and it is only alphanumeric
    if (strlen($token) !== 64 || !preg_match('/^[a-zA-Z0-9]+$/', $token)) {
        return adtrack_set_flash_message('The token you entered is invalid, token must have 64 chars and could contain only alphanumeric chars', 'error');
    }

    // get the old value from WP DB
    $oldToken = get_option(ADTRACK_API_TOKEN, '');

    // get the old security code value from WP DB
    $oldSecurityCode = get_option(ADTRACK_API_SECURITY_CODE, '');

    // Did we change any value ?
    if ($token === $oldToken && $oldSecurityCode === $securityCode) {
        return adtrack_set_flash_message('Nothing change!', 'error');
    }

    // Make the API to validate the token and retrieve the rental spaces
    $res = adtrack_api_call($token, $securityCode);
    // This should never get triggered
    if (!isset($res) || !is_array($res) || !isset($res['result-code'])) {
        return adtrack_set_flash_message('The was a problem validating your information', 'error');
    }
    // There was a problem with the call
    if ($res['result-code'] !== 'success') {
        return adtrack_set_flash_message(isset($res['result-code-string']) ? $res['result-code-string'] : 'Unknown error while validating your information', 'error');
    }

    if (!isset($res['components'])) {
        return adtrack_set_flash_message('The was a problem retrieving your information from AdTrack'. 'error');
    }
    $components = $res['components'];

    // Attempt to store token in database
    if ($token !== $oldToken && update_option(ADTRACK_API_TOKEN, $token, TRUE) === FALSE) {
        return adtrack_set_flash_message('Unable to store Access Key in database', 'error');
    }

    // Attempt to store token in database
    if ($oldSecurityCode !== $securityCode && update_option(ADTRACK_API_SECURITY_CODE, $securityCode, TRUE) === FALSE) {
        return adtrack_set_flash_message('Unable to store Security Code in database', 'error');
    }

    // Attempt to store token in database
    if (update_option(ADTRACK_APP_BOX_JSON, $components['rental_spaces_info'], TRUE) === FALSE) {
        return adtrack_set_flash_message('Unable to store the rental spaces in database'. 'error');
    }

    // store the time when we did this sync
    update_option(ADTRACK_LAST_SYNC, time(), TRUE);

    // form the string to store to file
    $confContent = '<?php
header("HTTP/1.1 403 Unauthorized");
exit();
/*
// Your API token.
api_token = ' . $token . '
// URL to the Adtrack API
api_url = https://api.adtrack.ca/api.php
*/
';
    if (file_put_contents(__DIR__ . '/adtrack.conf.php', $confContent) === FALSE) {
        return adtrack_set_flash_message('Unable to save API token to disk', 'error');
    }

    adtrack_set_flash_message('The Access Key and Security Code was verified and updated successfully!', 'success');
    return TRUE;
}

//! helper function that will refresh the rental spaces list by calling the AdTrack services
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_refresh_rental_list() {
    // get the old value from WP DB
    $token = get_option(ADTRACK_API_TOKEN, '');

    // get the old security code value from WP DB
    $securityCode = get_option(ADTRACK_API_SECURITY_CODE, '');

    // Make the API to validate the token and retrieve the rental spaces
    $res = adtrack_api_call($token, $securityCode);
    // This should never get triggered
    if (!isset($res) || !is_array($res) || !isset($res['result-code'])) {
        return adtrack_set_flash_message('The was a problem retrieving your information from AdTrack', 'error');
    }
    // There was a problem with the call
    if ($res['result-code'] !== 'success') {
        return adtrack_set_flash_message(isset($res['result-code-string']) ? $res['result-code-string'] : 'Unknown error while validating your information', 'error');
    }

    if (!isset($res['components'])) {
        return adtrack_set_flash_message('The was a problem retrieving your information from AdTrack', 'error');
    }
    $components = $res['components'];
    if (!isset($components['rental_spaces_info'])) {
        return adtrack_set_flash_message('The was a problem retrieving your information from AdTrack', 'error');
    }


    // get the rental spaces from the WP options
    $prevRentalSpaces = get_option(ADTRACK_APP_BOX_JSON, array());
    $suspendedIds = array();
    // if there are rental spaces let see what IDs are actually suspended
    // we want to keep the same option when we update the list
    if (is_array($prevRentalSpaces) && sizeof($prevRentalSpaces)) {
        foreach ($prevRentalSpaces as $space) {
            if (isset($space['suspend']) && $space['suspend'] === 1) {
                $suspendedIds[] = $space['app_box_id'];
            }
        }
    }

    // Determine if any of the rental spaces were suspended
    if (sizeof($components['rental_spaces_info']) > 0) {
        $changed = FALSE;
        foreach ($components['rental_spaces_info'] as $key => &$space) {
            if (in_array($space['app_box_id'], $suspendedIds)) {
                $space['suspend'] = 1;
            }
            // Are there any difference on this space like space Id
            if (!is_array($prevRentalSpaces) || !isset($prevRentalSpaces[$key]) || sizeof(array_diff_assoc($space, $prevRentalSpaces[$key])) > 0) {
                $changed = TRUE;
            }
        }
    }
    // If could be a size difference, mainly when we delete a record
    if (sizeof($components['rental_spaces_info']) !== sizeof($prevRentalSpaces)) {
        $changed = TRUE;
    }
    // store the time when we did this sync
    update_option(ADTRACK_LAST_SYNC, time(), TRUE);
    // Attempt to store token in database
    if ($changed === TRUE && update_option(ADTRACK_APP_BOX_JSON, $components['rental_spaces_info'], TRUE) === FALSE) {
        return adtrack_set_flash_message('Unable to store the updated Rental list in database', 'error');
    }
    adtrack_set_flash_message('AdTrack Rental Spaces list was updated successfully!', 'success');
    return TRUE;
}

//! helper function to add or update a AdTrack app box
/*!
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_process_update_app_boxes() {
    if (isset($_POST['update-list']) && $_POST['update-list'] == '1') {
        return adtrack_refresh_rental_list();
    }

    $suspendValue = 0;
    if (isset($_POST['enable']) && is_numeric($_POST['enable']) && $_POST['enable'] > 0) {
        $rentalId = intval($_POST['enable']);
    }

    if (isset($_POST['suspend']) && is_numeric($_POST['suspend']) && $_POST['suspend'] > 0) {
        $rentalId = intval($_POST['suspend']);
        $suspendValue = 1;
    }
    // This was neither a suspend or enable
    if (!isset($rentalId)) {
        return  adtrack_set_flash_message("Update rental spaces was called incorrectly", 'error');
    }
    $rentalSpaces = get_option(ADTRACK_APP_BOX_JSON, array());
    if (!isset($rentalSpaces) || !is_array($rentalSpaces) || sizeof($rentalSpaces) === 0) {
        return  adtrack_set_flash_message("You have no rental spaces", 'error');
    }
    $found = FALSE;
    foreach ($rentalSpaces as &$space) {
        if ($space['app_box_id'] == $rentalId) {
            $found = TRUE;
            $space['suspend'] = $suspendValue;
            // no need to go further
            break;
        }
    }

    if (!$found) {
        return  adtrack_set_flash_message("Unable to find the space that you want to update", 'error');
    }

    // Attempt to store token in database
    if (update_option(ADTRACK_APP_BOX_JSON, $rentalSpaces, TRUE) === FALSE) {
        return  adtrack_set_flash_message('Unable to update the rental space list in database', 'error');
    }
    $suspendString = $suspendValue === 0 ? 'enabled' : 'suspended';
    return adtrack_set_flash_message('The rental space was successfully ' . $suspendString, 'success');
}

// ****************************************************************************************************** //
// HTML Fragments Section
// ****************************************************************************************************** //

//! helper function that will generate the HTML fragment form to dashboard
/*!
 * @return string with HTML
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function adtrack_fragment_dashboard() {
    $html = '';
    $html .= '                      <div id="adtrack-dashboard" class="row tab-content">' . "\n";
    $html .= '                          <h2>AdTrack Dashboard</h2>' . "\n";

    // Determine if we have set the Access Key if not let show a blurb about this
    $token = get_option(ADTRACK_API_TOKEN, '');
    // Is this a first visitor (Does not have the API token set)
    if (empty($token)) {
        $html .= '                          <h4>First time visitor</h4>' . "\n";
        $html .= '                          <p>Before you can user this plugin, you need to have an active account on AdTrack cloud services.</p>' . "\n";
        $html .= '                          <p>If you do not have one then please visit <a href="https://www.adtrack.ca/">AdTrack</a> to register and sign up.</p>' . "\n";

        $html .= '                          <p>If you are already an AdTrack member, the first thing you need to do is to enter the Access Key ';
        $html .='(also know as API Token) and the API security code.<br /><strong>Enter your <a href="#"  onclick="AT.showBox(\'adtrack-api-form\')">Access Key and Security Code</a></strong>.' . "\n";
        $html .= '<br /><sup><em>Note: You can find the Access Key and Security Code on your AdTrack account, Dashboard / Space / Embedding in your site tab</em></sup></p>' . "\n";
    } else {
        // The APi token is set
        $html .= '                          <h4>Quick Starting instruction</h4>' . "\n";
        $html .= '                          <p>After you have added the Access key and Security Code, you can go to the <em>Rental Spaces</em> tab and see all the rental ';
        $html .= 'spaces that you can embed in your posts and/or pages.<br />' . "\n";

        $html .= 'Anywhere you wish your Rental Spaces to appear simply add the Wordpress shortcode corresponding to the space you wish to rent/display example <pre>[AdTrack id=10]</pre></p>' . "\n";
        $html .= '                          <p>From the list of <em>Rental Spaces</em> you can see any/all shortcodes that are available via AdTrack. ';
        $html .= '<br /><sup><em>Note: If you wish to use the rental spaces on the widget text, please go to settings tab and enable this option.</em></sub></p> ';

        $html .= '                          <h4>Manual Embedding Instructions</h4> ';
        $html .= '                          <p>There is a possibility to embed the <em>Rental Spaces</em> on your templates as well, but then you need to call the short code function your self.</p> ';
        $html .= "                          <p>Example<pre>&lt;?php do_shortcode('[AdTrack id=10]'); ?&gt;</pre></p>\n";
    }
    $html .= '                          <h4>About AdTrack plugin</h4>' . "\n";
    $html .= '                          <p>This is the AdTrack plugin that enables you an easily way to implement rental spaces on your Wordpress Posts, Pages, Widgets, Menus and much more.</p>' . "\n";

    $html .= '                      </div><!-- div.row -->' . "\n";

    return $html;
}

//! helper function that will generate the HTML fragment form notification
/*!
 * @return string with HTML
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 *
 * Note: By default this form is hidden by the CSS rule
 */
function adtrack_fragment_notification() {
    $html = '';

    $html .= '              <div id="adtrack-notification" class="row tab-content">' . "\n";
    $html .= '                  <h2>AdTrack Notification Center</h2>' . "\n";

    // Determine if we have set the Access Key if not let show a blurb about this
    $token = get_option(ADTRACK_API_TOKEN, '');
    if (empty($token)) {
        $html .= '                          <p><strong>No Access Key set</strong></p>' . "\n";
        $html .= '                          <p>In order for the plugin to securly communicate with AdTrack your need to set the Access Key and Security Code please follow this <a href="#"  onclick="AT.showBox(\'adtrack-api-form\')">link</a></p>.' . "\n";
    }

    $notifications = get_option(ADTRACK_NOTIFICATION_JSON, '');
    $html .= '                  <form method="post" id="adtrack-manage-apps">' . "\n";
    $html .= '                      <input type="hidden" name="adtrack_action" value="adtrack-notification-update" />' . "\n";
    $html .= '                      <h1>Manage your Notifications</h1>' . "\n";

    $html .= '                      <div class="row">' . "\n";

    $html .= '                          <table class="widefat fixed">' . "\n";
    $html .= '                              <tr>' . "\n";
    $html .= '                                  <th title="The notification rank" class="notification-index">#</th>' . "\n";
    $html .= '                                  <th title="The notification description/message" class="notification-description">Description</th>' . "\n";
    $html .= '                                  <th title="The notification time, when it was last noticed" class="notification-date">Date</th>' . "\n";
    $html .= '                                  <th title="Action the you can take against the notification" class="notification-action">Action</th>' . "\n";
    $html .= '                              </tr>' . "\n";
    // DO we have any app_boxes
    if (empty($notifications)) {
        $html .= '                              <tr>' . "\n";
        $html .= '                                  <td colspan="4"><em>You have no notification at the moment</em></td>' . "\n";
        $html .= '                              </tr>' . "\n";
    } else {
        // sort the notification by time
        uasort($notifications, 'adtrack_cmp');
        $index = 1;
        foreach ($notifications as $boxId => $item) {
            $showUnread = '';
            if ($item['read'] === FALSE) {
                $showUnread = ' adtrack-unread';
            }
            $html .= '                              <tr class="' . $showUnread . '">' . "\n";
            $html .= '                                  <td>' . esc_html($index) . '</td>' . "\n";
            // This could include HTML 
            $html .= '                                  <td>' . $item ['message'] . '</td>' . "\n";
            // Thanks to WP now we need to add hte offset to get the admin timezone settings
            $time = $item['time'] + (get_option('gmt_offset') * 60 * 60);
            $html .= '                                  <td>' . esc_html(date('D, M j G:i', $time)) . '</td>' . "\n";
            $html .= '                                  <td>';
            // Is this suspended on WP or not, this does not have anything to do with the state on AdTrack
            if ($item['read'] === FALSE) {
                $html .= '<button type="submit" name="read" value="' . esc_html($boxId) . '" class="button button-primary">Mark as Read</button>' . "\n";
            } else {
                $html .= '<button type="submit" name="unread" value="' . esc_html($boxId) . '" class="button button-primary">Mark as Unread</button>' . "\n";
            }

            if (!isset($item['ignore']) || $item['ignore'] !== 1) {
                $html .= '<button type="submit" name="ignore" value="' . esc_html($boxId) . '" class="button button-primary">Ignore</button>' . "\n";
            } else {
                $html .= '<button type="button" title="You will not receive notification for this rental space" disabled="disabled" class="button button-secondary">Ignore</button>' . "\n";
            }

            $html .= '<button type="submit" name="delete" onclick="return confirm(\'Are you sure you want to delete this notification?\')"value="' . esc_html($boxId) . '" class="button button-secondary">Delete</button>' . "\n";
            $html .= '</td>';
            $html .= '                              </tr>' . "\n";
            $index++;
        }
    }
    $html .= '                          </table>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";
    $html .= '                 </form>' . "\n";
    $html .= '              </div><!-- div.row -->' . "\n";
    return $html;
}

//! helper function that will generate the HTML fragment form rental space list
/*!
 * @return string with HTML
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 *
 * Note: By default this form is hidden by the CSS rule
 */
function adtrack_fragment_rental_space() {
    $html = '';

    $app_boxes = get_option(ADTRACK_APP_BOX_JSON, '');
    $html .= '              <div id="adtrack-rental" class="row tab-content">' . "\n";
    $html .= '                  <h2>AdTrack Rental Spaces</h2>' . "\n";

    // Determine if we have set the Access Key if not let show a blurb about this
    $token = get_option(ADTRACK_API_TOKEN, '');
    if (empty($token)) {
        $html .= '                          <p><strong>No Access Key set</strong></p>' . "\n";
        $html .= '                          <p>In order for the plugin to securly communicate with AdTrack your need to set the Access Key and Security Code please follow this <a href="#"  onclick="AT.showBox(\'adtrack-api-form\')">link</a></p>.' . "\n";
    }

    $lastSync = get_option(ADTRACK_LAST_SYNC, FALSE);
    if ($lastSync === FALSE) {
        $lastSyncHTML = 'Unknown';
    } else {
        // adjust the time to the WP offset
        $lastSync += (get_option('gmt_offset') * 60 * 60);
        $lastSyncHTML = date_i18n('D, M j G:i', $lastSync);
    }
    $html .= '                  <form method="post" id="adtrack-manage-apps">' . "\n";
    $html .= '                      <input type="hidden" name="adtrack_action" value="adtrack-update-app-boxes" />' . "\n";
    $html .= '                      <button type="submit" name="update-list" value="1" class="button button-primary" title="Last sync was done on: ' . $lastSyncHTML . '">Refresh Rental Space list</button>' . "\n";
    $html .= '                      <h1>Manage your Rental Spaces</h1>' . "\n";

    $html .= '                      <div class="row">' . "\n";

    $html .= '                          <table class="widefat fixed">' . "\n";
    $html .= '                              <tr>' . "\n";
    $html .= '                                  <th class="rental-id-col" title="Rental Space Id, is your AdTrack rental space unique identifier">Rental Space Id</th>' . "\n";
    $html .= '                                  <th class="rental-space-col" title="The Rental Space Id, is used by AdTrack to find your space, for human friendly names are more easy to remember">Rental Space Friendly Name</th>' . "\n";
    $html .= '                                  <th title="The stats on the last 24 hour. These stats are from the last update" >Stats last 24 hr</th>' . "\n";
    $html .= '                                  <th title="Shortcode is what you need to copy on the pages and post where you want to display / rent your space">Shortcode</th>' . "\n";
    $html .= '                                  <th title="What is the status of the current Rental Space" >Status</th>' . "\n";
    $html .= '                                  <th title="Action that you could perform to a space" >Action</th>' . "\n";
    $html .= '                              </tr>' . "\n";
    // DO we have any app_boxes
    if (empty($app_boxes)) {
        $html .= '                              <tr>' . "\n";
        $html .= '                                  <td colspan="6"><em>You have no rental spaces at the moment</em></td>' . "\n";
        $html .= '                              </tr>' . "\n";
    } else {
        foreach ($app_boxes as $box) {
            $html .= '                              <tr>' . "\n";
            $html .= '                                  <td>' . esc_html($box['app_box_id']) . '</td>' . "\n";
            $html .= '                                  <td>' . esc_html($box['description']) . '</td>' . "\n";
            $html .= '                                  <td>' . esc_html($box['impressions']) . '</td>' . "\n";
            $html .= '                                  <td>[AdTrack id=' . esc_html($box['app_box_id']) . ']</td>' . "\n";
            $html .= '                                  <td>';
            // Is this suspended on WP or not, this does not have anything to do with the state on AdTrack
            if ($box['suspend'] && $box['suspend'] == 1) {
                $html .= '<span class="">Suspended</span>';
            } else {
                $html .= '<span class="">Active</span>';
            }
            $html .= '</td>';

            $html .= '                                  <td>';

            // Is this suspended on WP or not
            if ($box['suspend'] && $box['suspend'] == 1) {
                $html .= '<button type="submit" name="enable" value="' . esc_html($box['app_box_id']) . '" class="button button-primary">Enable</button>' . "\n";
            } else {
                $html .= '<button type="submit" name="suspend" value="' . esc_html($box['app_box_id']) . '" class="button button-secondary">Suspend</button>' . "\n";
            }

            $html .= '</td>' . "\n";
            $html .= '                              </tr>' . "\n";
        }
    }
    $html .= '                          </table>' . "\n";
    $html .= '                      <p class="alignright"><em>Lastest update: ' . htmlspecialchars($lastSyncHTML) .'</em></p>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";
    $html .= '                 </form>' . "\n";
    $html .= '              </div><!-- div.row -->' . "\n";
    return $html;
}

//! helper function that will generate the HTML fragment form help page
/*!
 * @return string with HTML
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 *
 * Note: By default this form is hidden by the CSS rule
 */
function adtrack_fragment_help() {
    $html = '';
    $html .= '                      <div id="adtrack-help" class="row tab-content">' . "\n";
    $html .= '                          <h2>AdTrack Help</h2>' . "\n";

    // Determine if we have set the Access Key if not let show a blurb about this
    $token = get_option(ADTRACK_API_TOKEN, '');
    // Is this a first visitor (Does not have the API token set)
    if (empty($token)) {
        $html .= '                          <h4>First time visitor</h4>' . "\n";
        $html .= '                          <p>Before you can user this plugin, you need to have an active account on AdTrack cloud services.</p>' . "\n";
        $html .= '                          <p>If you do not have one then please visit <a href="https://www.adtrack.ca/">AdTrack</a> to register and sign up.</p>' . "\n";

        $html .= '                          <p>If you are already an AdTrack member, the first thing you need to do is to enter the Access Key ';
        $html .='(also know as API Token) and the API security code.<br /><strong>Enter your <a href="#"  onclick="AT.showBox(\'adtrack-api-form\')">Access Key and Security Code here</a></strong>.' . "\n";
        $html .= '<br /><sup><em>Note: You can find the Access Key and Security Code on your AdTrack account, Dashboard / Space / Embedding in your site tab</em></sup></p>' . "\n";
    } else {

        $html .= '                          <h4>Congratulations!</h4>' . "\n";
        $html .= '                          <p>Now that you have your AdTrack Wordpress Plugin live and active, we can assume you already understand how to add the WordPress \'Short Code\' ';
        $html .= 'anywhere in your pages that you wish to display/offer <em>Rental Spaces</em> for advertising. At this point, feel free to experiment by logging into your <a href="https://www.adtrack.ca/">AdTrack account</a> ';
        $html .= ' and see all of the customizable things you can do with your rental spaces.</p>' . "\n";

        $html .= '                          <p>You can control the size of the rental spaces, manage your own banners, share the rental space between banners, and/or advertise your rental space for rent.</p>' . "\n";

        $html .= '                          <p>AdTrack make it easier for you to:</p>' . "\n";
        $html .= '                          <ul>' . "\n";
        $html .= '                              <li>Manage your own banners, no matter what size/shape/size you want to allow;</li>' . "\n";
        $html .= '                              <li>Track impressions and stats for visitors who see your rental spaces;</li>' . "\n";
        $html .= '                              <li>Rent directly to an advertiser, whether you want to only make it available to advertisers you know (share secret key);</li>' . "\n";
        $html .= '                              <li>Allow advertisers to find you, or even become part of an advertising space network;</li>' . "\n";
        $html .= '                          </ul>' . "\n";

        $html .= '                          <p>It\'s your space, you should choose how much it is worth, and how you charge for it.</p>' . "\n";

        $html .= '                          <p>And both you AND your advertisers can get statistics and reports directly from the AdTrack Web Service.</p>' . "\n";

        $html .= '                          <p>However, if you need further help, please visit the AdTrack website itself, and use our contact forms and help there.</p>' . "\n";

        $html .= '                          <p>If the problem is specific to installing and/or using the AdTrack WordPress plugin, information is also available on the AdTrack Website.</p>' ."\n";


        // The APi token is set
        $html .= '                          <h4>Quick Starting instruction</h4>' . "\n";
        $html .= '                          <p>After you have added the Access key and Security Code, you can go to the <em>Rental Spaces</em> tab and see all the rental ';
        $html .= 'spaces that you can embed in your posts and/or pages.<br />' . "\n";

        $html .= 'Anywhere you wish your Rental Spaces to appear simply add the Wordpress shortcode corresponding to the space you wish to rent/display example <pre>[AdTrack id=10]</pre></p>' . "\n";
        $html .= '                          <p>From the list of <em>Rental Spaces</em> you can see any/all shortcodes that are available via AdTrack. ';
        $html .= '<br /><sup><em>Note: If you wish to use the rental spaces on the widget text, please go to settings tab and enable this option.</em></sub></p> ';

        $html .= '                          <h4>Manual Embedding Instructions</h4> ';
        $html .= '                          <p>There is a possibility to embed the <em>Rental Spaces</em> on your templates as well, but then you need to call the short code function your self.</p> ';
        $html .= "                          <p>Example<pre>&lt;?php do_shortcode('[AdTrack id=10]'); ?&gt;</pre></p>\n";
    }

    $html .= '                          <h4>About AdTrack plugin</h4>' . "\n";
    $html .= '                          <p>This is the AdTrack plugin that enables you an easily way to implement rental spaces on your Wordpress Posts, Pages, Widgets, menus and much more.</p>' . "\n";

    $html .= '              </div><!-- div.row -->' . "\n";

    return $html;
}

//! helper function that will generate the HTML fragment form setting page
/*!
 * @return string with HTML
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 *
 * Note: By default this form is hidden by the CSS rule
 */
function adtrack_fragment_settings() {
    $html = '';
    $html .= '                      <div id="adtrack-settings" class="row tab-content">' . "\n";

    $html .= '                      <form method="post">' . "\n";
    $html .= '                          <h2>AdTrack Settings</h2>' . "\n";
    $html .= '                          <input type="hidden" name="adtrack_action" value="adtrack-settings" />' . "\n";
    $html .= '                          <p>AdTrack access key and the security key are the credentials that are used by the this plugin to safely communicate with AdTrack cloud services .</p>' . "\n";

    $html .= '                          <button type="button" onclick="AT.showBox(\'adtrack-api-form\')" class="button button-primary">Update API Token</button><br /><br />' . "\n";

    $html .= '                          <table class="widefat fixed">' . "\n";
    $html .= '                              <tr>' . "\n";
    $html .= '                                  <th class="">Option</th>' . "\n";
    $html .= '                                  <th class="">Value</th>' . "\n";
    $html .= '                              </tr>' . "\n";

    $html .= '                              <tr>' . "\n";
    $html .= '                                  <td class="">Where do you wish to show your rental spaces?</td>' . "\n";

    $options = get_option(ADTRACK_OPTION_SETTINGS, array());
    $checkedPostOnly = '';
    $checkedWidget = '';
    if (!isset($options) || !is_array($options) || (isset($options['shortcode']) && $options['shortcode'] !== TRUE)) {
        $checkedPostOnly = 'checked="checked"';
    } else {
        $checkedWidget = 'checked="checked"';
    }
    $html .= '                                  <td class=""><span class="width-190">Pages and Posts only</span> <input type="radio" ' . $checkedPostOnly . ' name="shortcode" value="0"><br />' . "\n";
    $html .= '<span class="width-190">Pages, Posts and Widgets</span> <input type="radio" ' . $checkedWidget . ' name="shortcode" value="1"></td>' . "\n";
    $html .= '                              </tr>' . "\n";

    $html .= '                          </table>' . "\n";

    $html .= '                          <br /><input class="button button-primary alignright" type="submit" value="Update Settings" />' . "\n";
    $html .= '                      </form>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";

    return $html;
}


// ****************************************************************************************************** //
// HTML Form Fragments Section
// ****************************************************************************************************** //

//! helper function that will generate the HTML fragment form to update API token
/*!
 * @return string
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 *
 * Note: By default this form is hidden by the CSS rule
 */
function adtrack_update_api_token_fragment() {
    $html = '';
    $token = get_option(ADTRACK_API_TOKEN, '');
    $securityCode = get_option(ADTRACK_API_SECURITY_CODE, '');
    if (!empty($securityCode)) {
        $parts = str_split($securityCode, 4);
        $securityCode = strtoupper(join('-', $parts));
    }
    $html .= '                  <form method="post" id="adtrack-api-form">' . "\n";
    $html .= '                      <h1>AdTrack Access Key</h1>' . "\n";

    $html .= '                      <input type="hidden" name="adtrack_action" value="adtrack-update-api-token" />' . "\n";
    // API token
    $html .= '                      <div class="row">' . "\n";
    $html .= '                          <label>Please enter your AdTrack Access Key <em>(also known as API token)</em>:</label>' . "\n";
    $html .= '                          <input type="text" name="api_token" placeholder="e.g 5f868ee02e9677e758d935e3c4d5cfd3088fbd6f728ee8a38e76b280d8cac222" value="' . esc_html($token) . '"/>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";

    // security code
    $html .= '                      <div class="row">' . "\n";
    $html .= '                          <label>' . htmlspecialchars("Please enter your AdTrack Security Code:") . '</label>' . "\n";
    $html .= '                          <input type="text" name="security_code" placeholder="e.g AAAA-BBBB-CCCC-DDDD" value="' . esc_html($securityCode) . '"/>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";

    $html .= '                      <div class="row">' . "\n";
    $html .= '                          <button type="submit" class="button button-primary button-large">Update</button>' . "\n";
    $html .= '                          <button type="button" class="button button button-large" onclick="AT.hideConfirmBox();">Cancel</button>' . "\n";
    $html .= '                      </div><!-- div.row -->' . "\n";
    $html .= '                 </form>' . "\n";

    return $html;
}

